package com.qq.xwvoicesdk;

import android.Manifest;
import android.annotation.SuppressLint;
import android.content.Context;
import android.content.pm.PackageManager;
import android.media.AudioFormat;
import android.media.AudioRecord;
import android.media.MediaRecorder;
import android.media.audiofx.AcousticEchoCanceler;
import android.util.Log;

import androidx.core.app.ActivityCompat;

public class AudioRecorder {
    private int audioSource = MediaRecorder.AudioSource.VOICE_COMMUNICATION;
    private int sampleRateInHz = 16000;
    private int channelConfig = AudioFormat.CHANNEL_IN_MONO;
    private int audioFormat = AudioFormat.ENCODING_PCM_16BIT;
    private int bufferSizeInBytes = AudioRecord.getMinBufferSize(sampleRateInHz, channelConfig, audioFormat);

    @SuppressLint("MissingPermission")
    private AudioRecord audioRecorder = new AudioRecord(audioSource,
            sampleRateInHz,
            channelConfig,
            audioFormat,
            bufferSizeInBytes);

    private AcousticEchoCanceler canceler;

    private RecordCallback recordCallback;

    AudioRecorder (RecordCallback callback) {
        recordCallback = callback;
    }
    void startRecord(Context context) {
        if (AudioRecord.STATE_INITIALIZED != audioRecorder.getState()) {
            audioRecorder.release();
            if (ActivityCompat.checkSelfPermission(context, Manifest.permission.RECORD_AUDIO) != PackageManager.PERMISSION_GRANTED) {
                // TODO: Consider calling
                //    ActivityCompat#requestPermissions
                // here to request the missing permissions, and then overriding
                //   public void onRequestPermissionsResult(int requestCode, String[] permissions,
                //                                          int[] grantResults)
                // to handle the case where the user grants the permission. See the documentation
                // for ActivityCompat#requestPermissions for more details.
                return;
            }
            audioRecorder = new AudioRecord(audioSource,
                    sampleRateInHz,
                    channelConfig,
                    audioFormat,
                    bufferSizeInBytes);
        }
        if (android.os.Build.VERSION.SDK_INT >= 16) {
            if (AcousticEchoCanceler.isAvailable()) {
                canceler = AcousticEchoCanceler.create(audioRecorder.getAudioSessionId());
                if (canceler != null) {
                    canceler.setEnabled(true);
                }
            }
        }
        startRecordThread();
    }

    private RecordTask mRecordTask = new RecordTask();

    private void startRecordThread() {
        // 开始录音
        try {
            audioRecorder.startRecording();
        } catch (Exception e) {
            e.printStackTrace();
//            CommonApplication.showToast("请打开权限中的录音再重启APP");
            return;
        }
        mRecordTask.startRecording();
        Thread thread = new Thread(mRecordTask);
        thread.start();

    }

    // 录音工作线程，需要自行适配
    private class RecordTask implements Runnable {

        private volatile boolean isRecording = false;
        private final Object recordObject = new Object();

        public boolean isRecording() {
            synchronized (recordObject) {
                return isRecording;
            }
        }

        public void startRecording() {
            synchronized (recordObject) {
                this.isRecording = true;
            }
        }

        public void stopRecording() {
            synchronized (recordObject) {
                this.isRecording = false;
            }
        }

        @Override
        public void run() {

            while (isRecording()) {
                byte buffer[] = new byte[bufferSizeInBytes / 2];
                int read_len = audioRecorder.read(buffer, 0, buffer.length);
                if (read_len <= 0) {
                    // 录音出错了,一般是被别的程序占用了麦克风
                    Log.e("Record", "err");
                    break;
                }
                //抓到录音数据
                if (recordCallback != null) {
                    recordCallback.RecordCallback(buffer);
                }
//                RecordDataManager.getInstance().feedData(buffer);// buffer长度必须是32的倍数，建议每次传入的buffer大小在640-2048之间，不能超过6400。识别非常慢可以检查这个值的范围。
            }
            audioRecorder.stop();
        }
    }


    public void stopRecording() {

        if (android.os.Build.VERSION.SDK_INT >= 16 && AcousticEchoCanceler.isAvailable()) {
            if (canceler != null) {
                canceler.setEnabled(false);
                canceler.release();
            }
        }
        if (null != audioRecorder) {
            mRecordTask.stopRecording();
        }
    }

    interface RecordCallback{
        void RecordCallback(byte[] buffer);
    };
}
